import { Config, ConfigOptions, Plugin } from 'cli-engine-config';
import { HTTP } from 'http-call';
import { CLI } from 'cli-ux';
import { IArg } from 'cli-flags';
export interface CommandClass<T extends Command> {
    new ({config}: {
        config?: ConfigOptions;
    }): T;
}
export declare function run<T extends Command>(Command: CommandClass<T>, config?: ConfigOptions): Promise<T>;
export declare class Command {
    static topic: string;
    static command: string | undefined;
    static description: string | undefined;
    static hidden: boolean;
    static usage: string | undefined;
    static help: string | undefined;
    static aliases: string[];
    static variableArgs: boolean;
    static flags: {
        [name: string]: any;
    };
    static args: IArg[];
    static _version: any;
    static plugin: Plugin | undefined;
    static readonly id: string;
    /**
     * instantiate and run the command setting {mock: true} in the config (shorthand method)
     */
    static mock(...argv: string[]): Promise<Command>;
    /**
     * instantiate and run the command
     */
    static run(config?: ConfigOptions): Promise<Command>;
    readonly ctor: typeof Command;
    config: Config;
    http: typeof HTTP;
    cli: CLI;
    out: CLI;
    flags: any;
    argv: string[];
    args: {
        [name: string]: string;
    };
    constructor(options?: {
        config?: ConfigOptions;
    });
    init(): Promise<void>;
    topic: null;
    command: null;
    description: null;
    hidden: null;
    usage: null;
    help: null;
    aliases: null;
    /**
     * actual command run code goes here
     */
    run(): Promise<void>;
    readonly stdout: string;
    readonly stderr: string;
    static buildHelp(config: Config): string;
    static buildHelpLine(config: Config): [string, string | undefined];
}
