"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pjson = require('../package.json');
const cli_engine_config_1 = require("cli-engine-config");
const http_call_1 = require("http-call");
const help_1 = require("./help");
const cli_ux_1 = require("cli-ux");
const cli_flags_1 = require("cli-flags");
async function run(Command, config) {
    const cmd = new Command({ config });
    try {
        await cmd.init();
        await cmd.run();
        await cmd.out.done();
    }
    catch (err) {
        cmd.out.error(err);
    }
    return cmd;
}
exports.run = run;
class Command {
    constructor(options = {}) {
        this.args = {};
        this.config = cli_engine_config_1.buildConfig(options.config);
        this.argv = this.config.argv;
        const { cli } = require('cli-ux');
        this.out = this.cli = cli;
        this.out.color = require('./color').color;
        this.http = http_call_1.HTTP.defaults({
            headers: {
                'user-agent': `${this.config.name}/${this.config.version} (${this.config.platform}-${this.config.arch}) node-${process.version}`,
            },
        });
    }
    static get id() {
        let cmd = [];
        if (this.topic)
            cmd.push(this.topic);
        if (this.command)
            cmd.push(this.command);
        return cmd.join(':');
    }
    /**
     * instantiate and run the command setting {mock: true} in the config (shorthand method)
     */
    static async mock(...argv) {
        cli_ux_1.cli.warn('command.mock() is deprecated. Use cli-engine-test');
        argv.unshift('cmd');
        return this.run({ argv, mock: true });
    }
    /**
     * instantiate and run the command
     */
    static run(config) {
        return run(this, config);
    }
    get ctor() {
        return this.constructor;
    }
    async init() {
        const { argv, flags, args } = await cli_flags_1.parse({
            flags: this.ctor.flags || {},
            argv: this.argv.slice(1),
            args: this.ctor.args || [],
            strict: !this.ctor.variableArgs,
        });
        this.flags = flags;
        this.argv = argv;
        this.args = args;
    }
    /**
     * actual command run code goes here
     */
    async run() { }
    get stdout() {
        return this.out.stdout.output;
    }
    get stderr() {
        return this.out.stderr.output;
    }
    static buildHelp(config) {
        let help = new help_1.default(config);
        return help.command(this);
    }
    static buildHelpLine(config) {
        let help = new help_1.default(config);
        return help.commandLine(this);
    }
}
Command.aliases = [];
Command.variableArgs = false;
Command.args = [];
Command._version = pjson.version;
exports.Command = Command;
