import { Config } from 'cli-engine-config';
import { IBooleanFlag, IRequiredFlag, IOptionalFlag, IMultiOptionFlag, IOptionFlag, FlagBuilder } from 'cli-flags';
export { IBooleanFlag, IRequiredFlag, IOptionalFlag, IMultiOptionFlag };
export declare type CompletionContext = {
    args?: {
        [name: string]: string;
    };
    flags?: {
        [name: string]: string;
    };
    argv?: string[];
    config: Config;
};
export declare type Completion = {
    skipCache?: boolean;
    cacheDuration?: number;
    cacheKey?: (ctx: CompletionContext) => Promise<string>;
    options: (ctx: CompletionContext) => Promise<string[]>;
};
export declare type withCompletion = {
    completion?: Completion;
};
export declare type CompletionOptionFlag<T> = IOptionFlag<T> & withCompletion;
export declare type FlagBuilder<T> = {
    (options: Partial<IMultiOptionFlag<T>> & {
        multiple: true;
    }): IMultiOptionFlag<T> & withCompletion;
    (options: Partial<IRequiredFlag<T>> & {
        required: true;
    }): IRequiredFlag<T> & withCompletion;
    (options?: Partial<IOptionalFlag<T>>): IOptionalFlag<T> & withCompletion;
};
export declare const flags: {
    option: <T = string>(defaults?: Partial<IOptionalFlag<T> & withCompletion> | Partial<IRequiredFlag<T> & withCompletion> | Partial<IMultiOptionFlag<T> & withCompletion>) => FlagBuilder<T>;
    boolean: (options?: Partial<IBooleanFlag> | undefined) => IBooleanFlag;
    integer: FlagBuilder<number>;
    string: FlagBuilder<string>;
};
